@@ -149,8 +149,9 @@ destroy (HTMLObject *o)
 {
 	HTMLImage *image = HTML_IMAGE (o);
 
-	html_image_factory_unregister (image->image_ptr->factory,
-				       image->image_ptr, HTML_IMAGE (image));
+	if (image->image_ptr->factory)
+		html_image_factory_unregister (image->image_ptr->factory,
+					       image->image_ptr, HTML_IMAGE (image));
 
 	g_free (image->url);
 	g_free (image->target);
@@ -598,14 +599,14 @@ save (HTMLObject *self,
 	if (image->url && *image->url) {
 		url  = g_strconcat (image->url, image->target ? "#" : "", image->target, NULL);
 		link = TRUE;
-		result = html_engine_save_output_string (state, "<A HREF=\"%s\">", url);
+		result = html_engine_save_delims_and_vals (state, "<A HREF=\"", url, "\">", NULL);
 		g_free (url);
 		if (!result)
 			return FALSE;
 	}
 
 	url    = html_image_resolve_image_url (state->engine->widget, image->image_ptr->url);
-	result = html_engine_save_output_string (state, "<IMG SRC=\"%s\"", url);
+	result = html_engine_save_delims_and_vals (state, "<IMG SRC=\"", url, "\"", NULL);
 	g_free (url);
 	if (!result)
 		return FALSE;
@@ -647,7 +648,7 @@ save (HTMLObject *self,
 	}
 
 	if (image->alt) {
-		if (!html_engine_save_output_string (state, " ALT=\"%s\"", image->alt))
+		if (!html_engine_save_delims_and_vals (state, " ALT=\"", image->alt, "\"", NULL))
 			return FALSE;
 	}
 
@@ -1060,6 +1061,10 @@ update_or_redraw (HTMLImagePointer *ip){
 	GSList *list;
 	gboolean update = FALSE;
 
+	/* do nothing when factory gone; maybe the message drawing has been cancelled */
+	if (!ip->factory)
+		return;
+
 	for (list = ip->interests; list; list = list->next) {
 		if (list->data == NULL)
 			update = TRUE;
@@ -1120,12 +1125,16 @@ html_image_factory_end_pixbuf (GtkHTMLStream *stream,
 	g_object_unref (ip->loader);
 	ip->loader = NULL;
 
-	update_or_redraw (ip);
-	if (ip->factory->engine->opened_streams && ip->factory->engine->block_images)
-		html_engine_opened_streams_decrement (ip->factory->engine);
-	/* printf ("IMAGE(%p) opened streams: %d\n", ip->factory->engine, ip->factory->engine->opened_streams); */
-	if (ip->factory->engine->opened_streams == 0 && ip->factory->engine->block && ip->factory->engine->block_images)
-		html_engine_schedule_update (ip->factory->engine);
+	/* if no ip->factory is set, then the image loading has been cancelled meanwhile, probably. */
+	if (ip->factory) {
+		update_or_redraw (ip);
+		if (ip->factory->engine->opened_streams && ip->factory->engine->block_images)
+			html_engine_opened_streams_decrement (ip->factory->engine);
+		/* printf ("IMAGE(%p) opened streams: %d\n", ip->factory->engine, ip->factory->engine->opened_streams); */
+		if (ip->factory->engine->opened_streams == 0 && ip->factory->engine->block && ip->factory->engine->block_images)
+			html_engine_schedule_update (ip->factory->engine);
+	}
+
 	html_image_pointer_unref (ip);
 }
 
@@ -1160,10 +1169,12 @@ html_image_pointer_queue_animation (HTMLImagePointer *ip)
 static gint
 html_image_pointer_update (HTMLImagePointer *ip)
 {
-	HTMLEngine *engine = ip->factory->engine;
+	HTMLEngine *engine;
 	GSList *cur;
 
 	g_return_val_if_fail (ip->factory != NULL, FALSE);
+
+	engine = ip->factory->engine;
 	ip->animation_timeout = 0;
 
 	DA (printf ("animation_timeout (%p)\n", ip);)
@@ -1396,7 +1407,7 @@ html_image_pointer_unref (HTMLImagePointer *ip)
 static GtkHTMLStream *
 html_image_pointer_load (HTMLImagePointer *ip)
 {
-	if (ip->factory->engine->stopped)
+	if (!ip->factory || ip->factory->engine->stopped)
 		return NULL;
 
 	html_image_pointer_ref (ip);
